xquery version "3.0";

declare namespace bibo="http://purl.org/ontology/bibo/";
declare namespace dc="http://purl.org/dc/elements/1.1/";
declare namespace dcterms="http://purl.org/dc/terms/";
declare namespace gndo="https://d-nb.info/standards/elementset/gnd";
declare namespace rdf="https://www.w3.org/1999/02/22-rdf-syntax-ns";
declare namespace marc21="http://www.loc.gov/MARC21/slim";
declare namespace owl="http://www.w3.org/2002/07/owl#";
declare namespace rdagr1="http://rdvocab.info/Elements/";
declare namespace http="http://expath.org/ns/http-client";


declare option exist:serialize "method=json media-type=application/json";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../modules/config.xqm";
import module namespace console="http://exist-db.org/xquery/console";

let $id := request:get-parameter("id", "textgrid:24fp4")
let $objUri := 'http://textgridrep.de/' || $id
let $doc := collection($config:app-root || "/textgrid/rdf/")//rdf:Description[@rdf:about eq $objUri]
let $uri := ($doc//owl:sameAs)[1]/@rdf:resource/string()

let $target :=
    if(contains($uri, "gateway-bayern"))
        then concat('http://lod.b3kat.de/data/title/', substring-after($uri, 'http://gateway-bayern.de/'), '?output=xml')
    else if(contains($uri, "gso.gbv.de"))
        then concat('http://unapi.gbv.de/?id=gvk:ppn:', substring-after($uri, '?PPN='), '&#38;format=marcxml')
    else
        ""
let $req :=
    if($target)
        then <http:request href="{$target}" method="get">
                <http:header name="Connection" value="close"/>
           </http:request>
    else()

let $result :=
    if($req)
        then http:send-request($req)[2]
    else()

let $link :=
    if(contains($uri, "gateway-bayern"))
        then concat('http://lod.b3kat.de/title/', substring-after($uri, 'http://gateway-bayern.de/'))
    else
        ($doc//owl:sameAs)[1]/@rdf:resource/string()


let $datafield := $result//marc21:datafield

let $b3TitleTmp := ($result//dc:title)[1]/string()
let $b3Title := replace($b3TitleTmp, "[^\w\d\s']", "")
let $gbvTitle := $datafield[@tag = '245']/marc21:subfield[@code = 'a']/string()


(: solution for b3kat, where authors are connected with dnb norm database :)
let $b3AuthorArray :=
    if($result//dcterms:contributor)
        then $result//dcterms:contributor/@rdf:resource/string()
    else if($result//dcterms:creator)
        then $result//dcterms:creator/@rdf:resource/string()
    else ()

let $b3AuthNo := count($b3AuthorArray)


(: solution for gso.gvb.de, where authors are simply denoted as xs:string :)
let $gbvAuthorArray :=
    if($datafield[@tag = '100'])
        then $datafield[@tag = '100']/marc21:subfield[@code = 'a']/string()
    else if($datafield[@tag = '700'])
        then $datafield[@tag = '700']/marc21:subfield[@code = 'a']/string()
    else ()

let $gbvAuthNo := count($gbvAuthorArray)


let $b3Place := $result//rdagr1:placeOfPublication/string()

(:  bringing two year declarations in the right numeric order :)
let $issued := $result//dcterms:issued
let $b3Year := if(count($issued) = 2 and ($issued)[1] > ($issued)[2])
                    then
                        concat(($issued)[2]/string(), ', ', ($issued)[1]/string())
                else if(count($issued) = 2 and ($issued)[1] > ($issued)[2])
                    then
                        concat(($issued)[1]/string(), ', ', ($issued)[2]/string())
                else if($issued/string())
                    then concat(' ', $issued/string())
                else ""

(: due to unknown reasons tcincat is in some cases impossible and
 : leads to HTTP status 400 (no JSON is delivered), therefore string-join :)
let $b3PubStmt := string-join(($b3Place, $b3Year), ' ')


(: GBV place (a) and year (c) of publication :)
(: @ind1 = '3' necessary since in some data sets the place of publication is declared twice :)

let $gbvPubArray := $datafield[@tag = '264' and @ind1 = '3']
let $gbvPubArrayEntries := count($gbvPubArray)

(:DBG:)
(:
let $bla := console:log($gbvPubArrayEntries)

let $bla := for $iii in 1 to $gbvPubArrayEntries
                return
                console:log(string-join(
                ($gbvPubArray[$iii]/marc21:subfield[@code = 'a']/string(),
                $gbvPubArray[$iii]/marc21:subfield[@code = 'c']/string()), ' '))
:)

let $gbvPubPlaces := $datafield[@tag = '264' and @ind1 = '3']/marc21:subfield[@code = 'a']/string()
let $gbvPubYear := $datafield[@tag = '264' and @ind1 = '3']/marc21:subfield[@code = 'c']/string()
let $gbvPubStmt :=
    if(count($gbvPubPlaces) gt 1)
        then
            for $iii in 1 to count($gbvPubPlaces)
                let $place :=
                    if($iii != count($gbvPubPlaces))
                        then concat($gbvPubPlaces[$iii], ', ')
                        else concat($gbvPubPlaces[$iii], ' ', $gbvPubYear)
                return
                    $place
    else concat($gbvPubPlaces, ' ', $gbvPubYear)

return
    <data>
        <book>
            <title>
                {$b3Title}
                {$gbvTitle}
                {
                    if(contains($uri, 'swb'))
                        then string('ACHTUNG: Derzeit bietet der SWB keinen
                        Online-Zugriff auf seinen Linked Data Service an. Für
                        weitere Informationen zum Titel rufen Sie daher bitte
                        die unten stehende URL auf.')
                    else ""
                }
            </title>
            <b3>
                {
                    for $iii in 1 to $b3AuthNo
                        (: in the B3 data it sometimes is the case that a DNB ID
                        of an author starts with zero. This is an error due to
                        changes in the DNB and results in an 404. Therefore
                        the leading zero has to be purged. :)

                        let $authorTmp := $b3AuthorArray[$iii]
                        let $authorID := substring-after($authorTmp, "gnd/")
                        let $author := if(starts-with($authorID, '0'))
                            then replace($authorTmp, 'gnd/0', 'gnd/')
                            else $b3AuthorArray[$iii]
                        let $authorURI := concat($author, '/about/lds.rdf')

                        let $req :=
                            if($authorURI)
                                then <http:request href="{$authorURI}" method="get">
                                    <http:header name="Connection" value="close"/>
                                    </http:request>
                                else()
                        let $result :=
                            if($req)
                                then http:send-request($req)[2]
                            else()

                        let $namePath := $result//gndo:preferredNameEntityForThePerson/rdf:Description
                        let $namePrefix :=
                            if($namePath/gndo:prefix/string() = 'von')
                                then concat($namePath/gndo:prefix/string(), " ")
                            else if($namePath/gndo:prefix/string())
                                then $namePath/gndo:prefix/string()
                            else
                                ""

                        let $name :=
                            if($namePath/gndo:forename/string() != '')
                                then concat($namePath/gndo:forename/string(), " ", $namePrefix, $namePath/gndo:surname/string())
                            else if($namePath)
                                then concat($namePath/gndo:personalName/string(), " ", $namePath/gndo:epithetGenericNameTitleOrTerritory/string())
                            else
                                ""
                        return
                            <author>
                            <!-- inserting a blank space if entry is not the first one -->
                                {if($iii != 1)
                                    then concat(' ', $name)
                                    else
                                        $name}
                            </author>
                }
            </b3>

            <gbv>
                {
                    for $iii in 1 to $gbvAuthNo
                        let $author := $gbvAuthorArray[$iii]
                        let $forename :=
                            if(contains($author, ','))
                                then substring-after($author, ', ')
                            else()
                        let $surname :=
                            if(contains($author, ','))
                                then substring-before($author, ',')
                            else()
                        let $name :=
                            if($forename and $surname)
                                then concat($forename, ' ', $surname)
                            else $author
                        return
                            <author>
                            <!-- inserting a blank space if entry is not the first one -->
                                {if($iii != 1)
                                    then concat(' ', $name)
                                    else
                                        $name}
                            </author>
                }
            </gbv>

            <pubStmt>
                {$b3PubStmt}
                {
                    for $iii in 1 to $gbvPubArrayEntries
                return
                string-join(
                ($gbvPubArray[$iii]/marc21:subfield[@code = 'a']/string(),
                $gbvPubArray[$iii]/marc21:subfield[@code = 'c']/string()), ' ')
                }
            </pubStmt>

            <edition>
                {$result//bibo:edition/string()}
                {$datafield[@tag = '250']/marc21:subfield[@code = 'a']/string()}
            </edition>

            <publisher>
                {if(contains($result//rdagr1:publicationStatement/string(), ':'))
                    then substring-after($result//rdagr1:publicationStatement/string(), ':')
                else
                    $result//rdagr1:publicationStatement/string()}
                {$datafield[@tag = '260' and @ind1 = '3']/marc21:subfield[@code = 'b']/string()}
            </publisher>

            <uri>
                {$uri}
            </uri>
        </book>
    </data>
